package RDF::Server::Semantic;

use Moose::Role;

1;

__END__

=pod

=head1 NAME

RDF::Server::Semantic

=head1 AUTHOR 
            
James Smith, C<< <jsmith@cpan.org> >>
      
=head1 LICENSE
    
Copyright (c) 2008  Texas A&M University.
    
This library is free software.  You can redistribute it and/or modify
it under the same terms as Perl itself.
            
=cut
